//
//  LeapView.m
//  leapCinderObjC
//
//  Created by Nahuel Proietto on 16/08/13.
//  Copyright (c) 2013 ADBAND. All rights reserved.
//
//  leap's addons based on Stephen Schieberl's c++ block

/*
 *
 * Copyright (c) 2013, Ban the Rewind
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of the Ban the Rewind nor the names of its
 * contributors may be used to endorse or promote products
 * derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#import "LeapView.h"

@implementation LeapView

-(void)setup {
    
	[super setup];
    
    controller = [[LeapController alloc] init];
    [controller addListener:self];
    
    // camera setup
	CameraPersp cam;
    cam = CameraPersp([self getWindowWidth], [self getWindowHeight], 60.0f, 0.01f, 1000.0f);
	cam.lookAt(Vec3f(0.0f, 93.75f, 250.0f), Vec3f(0.0f, 250.0f, 0.0f));

	mMayaCam.setCurrentCam(cam);
    
    gl::enable( GL_LINE_SMOOTH );
	glHint( GL_LINE_SMOOTH_HINT, GL_NICEST );
	gl::enable( GL_POLYGON_SMOOTH );
	glHint( GL_POLYGON_SMOOTH_HINT, GL_NICEST );
    
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_COMPARE_R_TO_TEXTURE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

}

-(void)draw {
    
    float headLength = 6.0f;
	float headRadius = 3.0f;
    
    // clear window
    gl::setMatrices(mMayaCam.getCamera());
	gl::setViewport( [self getWindowBounds] );
	gl::clear( Colorf::white() );
    
	// depth enabled
	gl::enableAlphaBlending();
	gl::enableDepthRead();
	gl::enableDepthWrite();
    
    for (int i = 0; i < [[mFrame hands]count]; i++) {

        LeapHand *hand = [[mFrame hands] objectAtIndex:i];
        
        float sphereRadius	= [hand sphereRadius];
        
		// hand data
		Vec3f handDir = Vec3f( [[hand direction]toVec3f]);
		Vec3f palmNorm = Vec3f( [[hand palmNormal]toVec3f]);
		Vec3f palmPos = Vec3f( [[hand palmPosition]toVec3f]);
		Vec3f palmVel = Vec3f( [[hand palmVelocity]toVec3f]);
		Vec3f sphereCenter = Vec3f( [[hand sphereCenter]toVec3f]);
        
        // hand sphere
		gl::enableWireframe();
		gl::color( ColorAf( Colorf::gray( 0.9f ), 0.5f ) );
		gl::drawSphere( sphereCenter, sphereRadius, 16 );
		gl::disableWireframe();
        
		// hand plane
		gl::color( ColorAf( 0.75f, 0.0f, 0.75f, 0.25f ) );
		gl::pushMatrices();
		gl::translate( palmPos );
		gl::rotate( Quatf( palmPos, handDir ) );
        
		for ( float i = 0.25f; i <= 1.0f; i += 0.25f ) {
			gl::drawStrokedCircle( Vec2f::zero(), sphereRadius * i, 16 );
		}
        
		gl::popMatrices();
        
		// hand direction
		gl::color( 1.0f, 0.0f, 1.0f, 0.5f );
		gl::drawVector( palmPos, palmPos + handDir * 30.0f, headLength, headRadius );
        
		// hand normal
		gl::color( 0.0f, 0.0f, 1.0f, 0.5f );
		gl::drawVector( palmPos, palmPos + palmNorm * 30.0f, headLength, headRadius );
        
		// hand velocity
		gl::color( 0.0f, 1.0f, 0.0f, 0.5f );
		gl::drawVector( palmPos, palmPos + palmVel * 0.05f, headLength, headRadius );
        
        NSArray *fingers = [hand fingers];
        
        if ([fingers count] != 0) {
            
            for (int i = 0; i < [fingers count]; i++) {
                
                LeapFinger *finger = [fingers objectAtIndex:i];
                
                bool isTool = [finger isTool];
                float length = [finger length];
                float width = [finger width];
                
                Vec3f dir = Vec3f( [[finger direction]toVec3f]);
                Vec3f tipPos = Vec3f( [[finger tipPosition]toVec3f]);
                Vec3f tipVel = Vec3f( [[finger tipVelocity]toVec3f]);
                Vec3f basePos = tipPos + dir * -length;

                // draw line representing pointable's length
                gl::color( ColorAf::gray( 0.3f ) );
                gl::drawLine( basePos, tipPos );
                
                // tip position
                Colorf color = isTool ? Colorf( 1.0f, 0.0f, 0.0f ) : Colorf::black();
                gl::color( color );
                gl::pushMatrices();
                gl::translate( tipPos );
                gl::drawStrokedCircle( Vec2f::zero(), width, 16 );
                gl::popMatrices();
                
                // finger velocity
                gl::color( 0.0f, 1.0f, 0.0f, 0.5f );
                gl::drawVector( tipPos, tipPos + tipVel * 0.05f, headLength, headRadius );
                
            }
            
        }

    }
    
}

-(void)reshape {
    [super reshape];
	CameraPersp cam;
	cam = mMayaCam.getCamera();
	cam.setAspectRatio([self getWindowAspectRatio]);
	mMayaCam.setCurrentCam(cam);
}

#pragma mark callbacks

-(void)onInit:(NSNotification *)notification {
    NSLog(@"Initialized");
}

-(void)onConnect:(NSNotification *)notification {
    NSLog(@"Connected");
    LeapController *aController = (LeapController *)[notification object];
    [aController enableGesture:LEAP_GESTURE_TYPE_CIRCLE enable:YES];
    [aController enableGesture:LEAP_GESTURE_TYPE_KEY_TAP enable:YES];
    [aController enableGesture:LEAP_GESTURE_TYPE_SCREEN_TAP enable:YES];
    [aController enableGesture:LEAP_GESTURE_TYPE_SWIPE enable:YES];
}

-(void)onDisconnect:(NSNotification *)notification {
    NSLog(@"Disconnected");
}

-(void)onExit:(NSNotification *)notification {
    NSLog(@"Exited");
}

-(void)onFrame:(NSNotification *)notification {
    LeapController *aController = (LeapController *)[notification object];
    LeapFrame *frame = [aController frame:0];
    mFrame = frame;
}

@end